@@ -94,6 +94,8 @@
 #' @param lambda.optimization.tolerance Tolerance parameter, a double between 0 and 1 that fixes how much precision is required by the optimization method: the smaller the parameter, the higher the accuracy.
 #' Used only if \code{lambda.selection.criterion='newton'} or \code{lambda.selection.criterion='newton_fd'}.
 #' Default value \code{lambda.optimization.tolerance=0.05}.
+#' @param inference.data.object An \code{\link{inferenceDataObject}} that stores all the information regarding inference over the linear parameters of the model. This parameter needs to be 
+#' consistent with \code{covariates}, otherwise will be discarded. If set and well defined, the function will have in output the inference results. It is suggested to create this object via \code{\link{inferenceDataObjectBuilder}} function, so that the object is guaranteed to be well defined.
 #' @return A list with the following variables:
 #' \itemize{
 #'    \item{\code{fit.FEM}}{A \code{FEM} object that represents the fitted spatial field.}
@@ -123,14 +125,25 @@
 #'          \item{\code{J_minima}}{A vector of the same length of lambda, containing the reached minima for each value of the smoothing parameter.}
 #'          \item{\code{variance.est}}{A vector which returns the variance estimates for the Generative Additive Models.}
 #'         }
+#'    \item{\code{inference}}{A list set only if a well defined [inferenceDataObject] is passed as parameter to the function; contains all inference outputs required:
+#'          \item{\code{p_values}}{list of lists set only if at least one p-value is required; contains the p-values divided by implementation:
+#'               \item{\code{wald}}{list containing all the Wald p-values required, in the same order of the  \code{type} list in \code{inference.data.object}. If one-at-the-time tests are required, the corresponding item is a vector of p values ordered as the rows of \code{coeff} matrix in \code{inference.data.object}.}
+#'               \item{\code{speckman}}{list containing all the Speckman p-values required, in the same order of the  \code{type} list in  \code{inference.data.object}. If one-at-the-time tests are required, the corresponding item is a vector of p values ordered as the rows of \code{coeff} matrix in \code{inference.data.object}.}
+#'               \item{\code{eigen_sign_flip}}{list containing all the Eigen-Sign-Flip p-values required, in the same order of the \code{type} list in \code{inference.data.object}. If one-at-the-time tests are required, the corresponding item is a vector of p values ordered as the rows of \code{coeff} matrix in \code{inference.data.object}.}
+#'               }
+#'         \item{\code{CI}}{list of lists set only if at least one confidence interval is required; contains the confidence intervals divided by implementation:
+#'               \item{\code{wald}}{list containing all the Wald confidence intervals required, in the same order of the  \code{type} list in \code{inference.data.object}. Each item is a matrix with 3 columns and p rows, p being the number of rows of \code{coeff} matrix in \code{inference.data.object}; each row is the CI for the corresponding row of \code{coeff} matrix.}
+#'               \item{\code{speckman}}{list containing all the Speckman confidence intervals required, in the same order of the  \code{type} list in \code{inference.data.object}. Each item is a matrix with 3 columns and p rows, p being the number of rows of \code{coeff} matrix in \code{inference.data.object}; each row is the CI for the corresponding row of \code{coeff} matrix.}
+#'              }
+#'         }
 #' }
 #' @description This function implements a spatial regression model with differential regularization.
 #'  The regularizing term involves a Partial Differential Equation (PDE). In the simplest case the PDE involves only the
 #'  Laplacian of the spatial field, that induces an isotropic smoothing. When prior information about the anisotropy or
 #'  non-stationarity is available the PDE involves a general second order linear differential operator with possibly
-#'  space-varying coefficients.
+#'  space-varying coefficients. 
 #'  The technique accurately handle data distributed over irregularly shaped domains. Moreover, various conditions
-#'  can be imposed at the domain boundaries.
+#'  can be imposed at the domain boundaries
 #' @usage smooth.FEM(locations = NULL, observations, FEMbasis,
 #'  covariates = NULL, PDE_parameters = NULL, BC = NULL,
 #'  incidence_matrix = NULL, areal.data.avg = TRUE,
@@ -139,9 +152,9 @@
 #'  max.steps.FPIRLS = 15, lambda.selection.criterion = "grid", DOF.evaluation = NULL, 
 #'  lambda.selection.lossfunction = NULL, lambda = NULL, DOF.stochastic.realizations = 100,
 #'  DOF.stochastic.seed = 0, DOF.matrix = NULL, GCV.inflation.factor = 1, 
-#'  lambda.optimization.tolerance = 0.05)
+#'  lambda.optimization.tolerance = 0.05,
+#'  inference.data.object=NULL)
 #' @export
-
 #' @references
 #' \itemize{
 #'    \item{Sangalli, L. M., Ramsay, J. O., Ramsay, T. O. (2013). Spatial spline regression models.
@@ -150,6 +163,8 @@
 #' via spatial regression with PDE penalization. Journal of the American Statistical Association, 110(511), 1057-1071.}
 #'    \item{Matthieu Wilhelm & Laura M. Sangalli (2016). Generalized spatial regression with differential regularization.
 #'  Journal of Statistical Computation and Simulation, 86:13, 2497-2518.}
+#'    \item{Federico Ferraccioli, Laura M. Sangalli & Livio Finos (2021). Some first inferential tools for spatial regression
+#'    with differential regularization. Journal of Multivariate Analysis, to appear}
 #' }
 #' @examples
 #' library(fdaPDE)
@@ -172,12 +187,20 @@
 #' # with covariates
 #' covariate = covs.test(mesh$nodes[,1], mesh$nodes[,2])
 #' data = fs.test(mesh$nodes[,1], mesh$nodes[,2]) + 2*covariate + rnorm(nrow(mesh$nodes), sd = 0.5)
+#' 
+#' #Inferential tests and confidence intervals
+#' inference.data.object = inferenceDataObjectBuilder(test = 'oat', type = 'w', dim = 2, n_cov = 1)
 #'
 #' solution = smooth.FEM(observations = data, covariates = covariate, 
-#'                       FEMbasis = FEMbasis, lambda = lambda)
+#'                       FEMbasis = FEMbasis, lambda = lambda,
+#'                       inference.data.object=inference.data.object)
 #'
 #' # beta estimate:
 #' solution$solution$beta
+#' # tests over beta estimates p-values:
+#' solution$inference$beta$p_values
+#' # confidence intervals for beta estimates:
+#' solution$inference$beta$CI
 #' # non-parametric estimate:
 #' plot(solution$fit.FEM)
 #'
@@ -198,7 +221,7 @@
 #'
 #'
 #' #### Smoothing with prior information about anysotropy/non-stationarity and boundary conditions ####
-#' # See Azzimonti et al. for reference to the current exemple
+#' # See Azzimonti et al. for reference to the current example
 #' data(quasicircle2D)
 #' boundary_nodes = quasicircle2D$boundary_nodes
 #' boundary_segments = quasicircle2D$boundary_segments
@@ -342,7 +365,8 @@ smooth.FEM<-function(locations = NULL, observations, FEMbasis,
                      search = "tree", bary.locations = NULL,
                      family = "gaussian", mu0 = NULL, scale.param = NULL, threshold.FPIRLS = 0.0002020, max.steps.FPIRLS = 15,
                      lambda.selection.criterion = "grid", DOF.evaluation = NULL, lambda.selection.lossfunction = NULL,
-                     lambda = NULL, DOF.stochastic.realizations = 100, DOF.stochastic.seed = 0, DOF.matrix = NULL, GCV.inflation.factor = 1, lambda.optimization.tolerance = 0.05)
+                     lambda = NULL, DOF.stochastic.realizations = 100, DOF.stochastic.seed = 0, DOF.matrix = NULL, GCV.inflation.factor = 1, lambda.optimization.tolerance = 0.05,
+                     inference.data.object=NULL)
 {
   # Mesh identification
   if(is(FEMbasis$mesh, "mesh.2D"))
@@ -496,7 +520,30 @@ smooth.FEM<-function(locations = NULL, observations, FEMbasis,
     search = search, bary.locations = bary.locations,
     optim = optim, lambda = lambda, DOF.stochastic.realizations = DOF.stochastic.realizations, DOF.stochastic.seed = DOF.stochastic.seed,
     DOF.matrix = DOF.matrix, GCV.inflation.factor = GCV.inflation.factor, lambda.optimization.tolerance = lambda.optimization.tolerance)
-
+  
+  # Checking inference data
+  # Most of the checks have already been carried out by inferenceDataObjectBuilder function
+  if(!is.null(locations))
+    inference.data.object <- checkInferenceParameters(inference.data.object,ncol(covariates),locations,FEMbasis$mesh$nodes) #checking inference data consistency, constructing default object in NULL case
+  else
+    inference.data.object <- checkInferenceParameters(inference.data.object,ncol(covariates),FEMbasis$mesh$nodes[1:length(observations),],FEMbasis$mesh$nodes)
+  
+  # Check that GCV is set for inference
+  if((inference.data.object@definition==1) && (is.null(lambda.selection.lossfunction)) && (!is.null(lambda)) && (ncol(lambda)!=1 || nrow(lambda)!=1)){
+    warning("Inference is not defined when lambda grid is provided without GCV, discarding inference")
+    inference.data.object=new("inferenceDataObject", test = as.integer(0), interval =as.integer(0), type = as.integer(0), component = as.integer(0), exact = as.integer(0), dim = as.integer(0), n_cov = as.integer(0), 
+                                locations = matrix(data=0, nrow = 1 ,ncol = 1), locations_indices = as.integer(0), locations_are_nodes = as.integer(0), coeff = matrix(data=0, nrow = 1 ,ncol = 1), beta0 = -1, f0 = function(){}, 
+                                f0_eval = -1, f_var = as.integer(0), quantile = -1, alpha = 0, n_flip = as.integer(1000), tol_fspai = -1, definition=as.integer(0))
+  }
+  
+  # Inference on f not implemented in the areal case
+  if((inference.data.object@definition==1) && (any(inference.data.object@component!=1)) && (!is.null(incidence_matrix))){
+    warning("Inference on f is not implemented for areal data, discarding inference")
+    inference.data.object=new("inferenceDataObject", test = as.integer(0), interval =as.integer(0), type = as.integer(0), component = as.integer(0), exact = as.integer(0), dim = as.integer(0), n_cov = as.integer(0), 
+                              locations = matrix(data=0, nrow = 1 ,ncol = 1), locations_indices = as.integer(0), locations_are_nodes = as.integer(0), coeff = matrix(data=0, nrow = 1 ,ncol = 1), beta0 = -1, f0 = function(){}, 
+                              f0_eval = -1, f_var = as.integer(0), quantile = -1, alpha = 0, n_flip = as.integer(1000), tol_fspai = -1, definition=as.integer(0))
+  }
+  
   # If I have PDE non-sv case I need (constant) matrices as parameters
   if(!is.null(PDE_parameters) & space_varying == FALSE)
   {
@@ -527,7 +574,7 @@ smooth.FEM<-function(locations = NULL, observations, FEMbasis,
       if (sum1==0 & sum2==0)
       {
         message("No search algorithm is used because the locations coincide with the nodes.")
-        locations = NULL #In principle, R uses pass-by-value semantics in its function calls. So put ouside of checkSmoothingParameters function.
+        locations = NULL #In principle, R uses pass-by-value semantics in its function calls. So put outside of checkSmoothingParameters function.
       }
     }
   }
@@ -538,7 +585,13 @@ smooth.FEM<-function(locations = NULL, observations, FEMbasis,
    stop("'family' parameter required.\nCheck if it is one of the following: binomial, exponential, gamma, poisson, gaussian")
   }
 
-
+  if(inference.data.object@definition==1 && (family!= "gaussian"))
+  {
+    warning("Inference for linear estimators is implemented only for gaussian family in regression-Laplace and regression-PDE,\nInference Data are ignored")
+    inference.data.object=new("inferenceDataObject", test = as.integer(0), interval =as.integer(0), type = as.integer(0), component = as.integer(0), exact = as.integer(0), dim = as.integer(0), 
+                                locations = matrix(data=0, nrow = 1 ,ncol = 1), locations_indices = as.integer(0), locations_are_nodes = as.integer(0), coeff = matrix(data=0, nrow = 1 ,ncol = 1), beta0 = -1, f0 = function(){}, 
+                                f0_eval = -1, f_var = as.integer(0), quantile = -1, alpha = 0, n_flip = as.integer(1000), tol_fspai = -1, definition=as.integer(0))
+  }
 
   ################## End checking parameters, sizes and conversion #############################
   if(family == "gaussian")
@@ -554,7 +607,8 @@ smooth.FEM<-function(locations = NULL, observations, FEMbasis,
         incidence_matrix = incidence_matrix, areal.data.avg = areal.data.avg,
         search = search, bary.locations = bary.locations,
         optim = optim, lambda = lambda, DOF.stochastic.realizations = DOF.stochastic.realizations, DOF.stochastic.seed = DOF.stochastic.seed,
-        DOF.matrix = DOF.matrix, GCV.inflation.factor = GCV.inflation.factor, lambda.optimization.tolerance = lambda.optimization.tolerance)
+        DOF.matrix = DOF.matrix, GCV.inflation.factor = GCV.inflation.factor, lambda.optimization.tolerance = lambda.optimization.tolerance,
+        inference.data.object=inference.data.object)
       numnodes = nrow(FEMbasis$mesh$nodes)
     }else if(is(FEMbasis$mesh, "mesh.2D") & !is.null(PDE_parameters) & space_varying == FALSE)
     {
@@ -564,7 +618,9 @@ smooth.FEM<-function(locations = NULL, observations, FEMbasis,
         incidence_matrix = incidence_matrix, areal.data.avg = areal.data.avg,
         search = search, bary.locations = bary.locations,
         optim = optim, lambda = lambda, DOF.stochastic.realizations = DOF.stochastic.realizations, DOF.stochastic.seed = DOF.stochastic.seed, 
-        DOF.matrix = DOF.matrix, GCV.inflation.factor = GCV.inflation.factor, lambda.optimization.tolerance = lambda.optimization.tolerance)
+        DOF.matrix = DOF.matrix, GCV.inflation.factor = GCV.inflation.factor, lambda.optimization.tolerance = lambda.optimization.tolerance,
+        inference.data.object=inference.data.object
+      )
       numnodes = nrow(FEMbasis$mesh$nodes)
     }else if(is(FEMbasis$mesh, "mesh.2D") & !is.null(PDE_parameters) & space_varying == TRUE)
     {
@@ -574,7 +630,8 @@ smooth.FEM<-function(locations = NULL, observations, FEMbasis,
         incidence_matrix=incidence_matrix, areal.data.avg = areal.data.avg,
         search=search, bary.locations = bary.locations,
         optim = optim, lambda = lambda, DOF.stochastic.realizations = DOF.stochastic.realizations, DOF.stochastic.seed = DOF.stochastic.seed,
-        DOF.matrix = DOF.matrix, GCV.inflation.factor = GCV.inflation.factor, lambda.optimization.tolerance = lambda.optimization.tolerance)
+        DOF.matrix = DOF.matrix, GCV.inflation.factor = GCV.inflation.factor, lambda.optimization.tolerance = lambda.optimization.tolerance,
+        inference.data.object=inference.data.object) # further check for inference consistency will be performed deeper in the code
       numnodes = nrow(FEMbasis$mesh$nodes)
     }else if(is(FEMbasis$mesh, "mesh.2.5D"))
     {
@@ -586,7 +643,8 @@ smooth.FEM<-function(locations = NULL, observations, FEMbasis,
        incidence_matrix = incidence_matrix, areal.data.avg = areal.data.avg,
        search = search, bary.locations = bary.locations,
        optim = optim, lambda = lambda, DOF.stochastic.realizations = DOF.stochastic.realizations, DOF.stochastic.seed = DOF.stochastic.seed,
-       DOF.matrix = DOF.matrix, GCV.inflation.factor = GCV.inflation.factor, lambda.optimization.tolerance = lambda.optimization.tolerance)
+       DOF.matrix = DOF.matrix, GCV.inflation.factor = GCV.inflation.factor, lambda.optimization.tolerance = lambda.optimization.tolerance,
+       inference.data.object=inference.data.object)
       numnodes = nrow(FEMbasis$mesh$nodes)
   	}else if(is(FEMbasis$mesh, "mesh.3D") & is.null(PDE_parameters))
     {
@@ -596,7 +654,9 @@ smooth.FEM<-function(locations = NULL, observations, FEMbasis,
         incidence_matrix = incidence_matrix, areal.data.avg = areal.data.avg,
         search = search, bary.locations = bary.locations,
         optim = optim, lambda = lambda, DOF.stochastic.realizations = DOF.stochastic.realizations, DOF.stochastic.seed = DOF.stochastic.seed,
-        DOF.matrix = DOF.matrix, GCV.inflation.factor = GCV.inflation.factor, lambda.optimization.tolerance = lambda.optimization.tolerance)
+        DOF.matrix = DOF.matrix, GCV.inflation.factor = GCV.inflation.factor, lambda.optimization.tolerance = lambda.optimization.tolerance,
+        inference.data.object=inference.data.object
+        )
       numnodes = nrow(FEMbasis$mesh$nodes)
     } else if(is(FEMbasis$mesh, "mesh.3D") & !is.null(PDE_parameters) & space_varying==FALSE)
     {
@@ -606,7 +666,9 @@ smooth.FEM<-function(locations = NULL, observations, FEMbasis,
         incidence_matrix = incidence_matrix, areal.data.avg = areal.data.avg,
         search = search, bary.locations = bary.locations,
         optim = optim, lambda = lambda, DOF.stochastic.realizations = DOF.stochastic.realizations, DOF.stochastic.seed = DOF.stochastic.seed,
-        DOF.matrix = DOF.matrix, GCV.inflation.factor = GCV.inflation.factor, lambda.optimization.tolerance = lambda.optimization.tolerance)
+        DOF.matrix = DOF.matrix, GCV.inflation.factor = GCV.inflation.factor, lambda.optimization.tolerance = lambda.optimization.tolerance,
+        inference.data.object=inference.data.object
+        )
       numnodes = nrow(FEMbasis$mesh$nodes)
   	} else if(is(FEMbasis$mesh, "mesh.3D") & !is.null(PDE_parameters) & space_varying==TRUE)
   	{
@@ -616,7 +678,9 @@ smooth.FEM<-function(locations = NULL, observations, FEMbasis,
         incidence_matrix = incidence_matrix, areal.data.avg = areal.data.avg,
         search = search, bary.locations = bary.locations,
         optim = optim, lambda = lambda, DOF.stochastic.realizations = DOF.stochastic.realizations, DOF.stochastic.seed = DOF.stochastic.seed,
-        DOF.matrix = DOF.matrix, GCV.inflation.factor = GCV.inflation.factor, lambda.optimization.tolerance = lambda.optimization.tolerance)
+        DOF.matrix = DOF.matrix, GCV.inflation.factor = GCV.inflation.factor, lambda.optimization.tolerance = lambda.optimization.tolerance,
+        inference.data.object=inference.data.object
+        ) # further check for inference consistency will be performed deeper in the code
       numnodes = nrow(FEMbasis$mesh$nodes)
   	} else if(is(FEMbasis$mesh, "mesh.1.5D"))
   	{
@@ -626,7 +690,8 @@ smooth.FEM<-function(locations = NULL, observations, FEMbasis,
   	                                      incidence_matrix = incidence_matrix, areal.data.avg = areal.data.avg,
   	                                      search = search, bary.locations = bary.locations,
   	                                      optim = optim, lambda = lambda, DOF.stochastic.realizations = DOF.stochastic.realizations, DOF.stochastic.seed = DOF.stochastic.seed,
-  	                                      DOF.matrix = DOF.matrix, GCV.inflation.factor = GCV.inflation.factor, lambda.optimization.tolerance = lambda.optimization.tolerance)
+  	                                      DOF.matrix = DOF.matrix, GCV.inflation.factor = GCV.inflation.factor, lambda.optimization.tolerance = lambda.optimization.tolerance,
+  	                                      inference.data.object = inference.data.object)
   	  numnodes = nrow(FEMbasis$mesh$nodes)
   	} 
   } else
@@ -932,8 +997,143 @@ smooth.FEM<-function(locations = NULL, observations, FEMbasis,
     fit.FEM  = FEM(solution$f, FEMbasis)
     PDEmisfit.FEM = FEM(solution$g, FEMbasis)
 
+    # Save statistics and intervals
+    if(inference.data.object@definition==1){
+      inference = {}
+      confidence_intervals = matrix(data = bigsol[[24]], nrow = 2*3*length(inference.data.object@type), ncol = max(dim(inference.data.object@coeff)[1], dim(inference.data.object@locations)[1]))
+      p_val = matrix(data = bigsol[[23]], nrow = dim(inference.data.object@coeff)[1]+1, ncol = length(inference.data.object@type))
+  
+      for(i in 1:length(inference.data.object@type)){ # each element is a different inferential setting
+        if(inference.data.object@interval[i]!=0){ # Intervals requested by this setting, adding them to the right implementation position
+          
+          if(inference.data.object@component[i]!=2){ # intervals for beta were requested
+            p = dim(inference.data.object@coeff)[1]
+            ci_beta=t(confidence_intervals[(3*(2*i-2)+1):(3*(2*i-2)+3),1:p])
+            if(inference.data.object@type[i]==1){
+              inference$beta$CI$wald[[length(inference$beta$CI$wald)+1]] = ci_beta
+              inference$beta$CI$wald=as.list(inference$beta$CI$wald)
+            }
+            else if(inference.data.object@type[i]==2){
+              inference$beta$CI$speckman[[length(inference$beta$CI$speckman)+1]] = ci_beta
+              inference$beta$CI$speckman=as.list(inference$beta$CI$speckman)
+            }
+            else if(inference.data.object@type[i]==3){
+              if(ci_beta[2]> 10^20){
+                warning("ESF CI bisection algorithm did not converge, returning NA")
+                for(h in 1:nrow(ci_beta))
+                  for(k in 1:ncol(ci_beta))
+                    ci_beta[h,k]=NA
+              }
+              inference$beta$CI$eigen_sign_flip[[length(inference$beta$CI$eigen_sign_flip)+1]] = ci_beta
+              inference$beta$CI$eigen_sign_flip=as.list(inference$beta$CI$eigen_sign_flip)
+            }
+            else if(inference.data.object@type[i]==4){
+              if(ci_beta[2]> 10^20){
+                warning("Enhanced ESF CI bisection algorithm did not converge, returning NA")
+                for(h in 1:nrow(ci_beta))
+                  for(k in 1:ncol(ci_beta))
+                    ci_beta[h,k]=NA
+              }
+              inference$beta$CI$enh_eigen_sign_flip[[length(inference$beta$CI$enh_eigen_sign_flip)+1]] = ci_beta
+              inference$beta$CI$enh_eigen_sign_flip=as.list(inference$beta$CI$enh_eigen_sign_flip)
+            }
+          }
+          if(inference.data.object@component[i]!=1){ # intervals for f were requested
+            n_loc = dim(inference.data.object@locations)[1]
+            ci_f=t(confidence_intervals[(3*(2*i-1)+1):(3*(2*i-1)+3),])
+            if(inference.data.object@type[i]==1){ # wald confidence intervals for f
+              inference$f$CI$wald[[length(inference$f$CI$wald)+1]] = ci_f
+              inference$f$CI$wald=as.list(inference$f$CI$wald)
+            }
+            else if(inference.data.object@type[i]==3){ # eigen-sign-flip confidence intervals for f
+              inference$f$CI$eigen_sign_flip[[length(inference$f$CI$eigen_sign_flip)+1]] = ci_f
+              inference$f$CI$eigen_sign_flip=as.list(inference$f$CI$eigen_sign_flip)
+            }
+            else if(inference.data.object@type[i]==5){ # sign-flip confidence intervals for f
+              inference$f$CI$sign_flip[[length(inference$f$CI$sign_flip)+1]] = ci_f
+              inference$f$CI$sign_flip=as.list(inference$f$CI$sign_flip)
+            }
+          }
+        }
+        
+        if(inference.data.object@test[i]!=0){ # Test requested by this setting, adding them to the right implementation position
+        statistics=p_val[,i]
+        if(inference.data.object@component[i]!=2){ # test on beta was requested
+        beta_statistics = statistics[1:dim(inference.data.object@coeff)[1]]
+        p_values = numeric()
+        if(inference.data.object@type[i]==3 || inference.data.object@type[i]==4){ # eigen-sign-flip p-value is already computed in cpp code
+          if(inference.data.object@test[i]==1){ 
+            # one-at-the-time tests
+            p_values = beta_statistics
+          }
+          else if(inference.data.object@test[i]==2){
+            # simultaneous test
+            p_values = beta_statistics[1]
+          }
+        }else{
+          # Compute p-values
+          if(inference.data.object@test[i]==1){ # Wald and Speckman return statistics and needs computation of p-values (no internal use of distributions quantiles)
+            # one-at-the-time-tests
+            p_values = numeric(length(beta_statistics))
+            for(l in 1:length(beta_statistics)){
+              p_values[l] = 2*pnorm(-abs(beta_statistics[l]))
+            }
+          }
+          else if(inference.data.object@test[i]==2){
+            # simultaneous tests
+            p = dim(inference.data.object@coeff)[1]
+            p_values = 1-pchisq(beta_statistics[1], p)
+          }
+        }
+        # add p-values in the right position
+        if(inference.data.object@type[i]==1){
+          inference$beta$p_values$wald[[length(inference$beta$p_values$wald)+1]] = p_values
+          inference$beta$p_values$wald=as.list(inference$beta$p_values$wald)
+        }
+        else if(inference.data.object@type[i]==2){
+          inference$beta$p_values$speckman[[length(inference$beta$p_values$speckman)+1]] = p_values
+          inference$beta$p_values$speckman=as.list(inference$beta$p_values$speckman)
+        }
+        else if(inference.data.object@type[i]==3){
+          inference$beta$p_values$eigen_sign_flip[[length(inference$beta$p_values$eigen_sign_flip)+1]] = p_values
+          inference$beta$p_values$eigen_sign_flip=as.list(inference$beta$p_values$eigen_sign_flip)
+        }
+        else if(inference.data.object@type[i]==4){
+          inference$beta$p_values$enh_eigen_sign_flip[[length(inference$beta$p_values$enh_eigen_sign_flip)+1]] = p_values
+          inference$beta$p_values$enh_eigen_sign_flip=as.list(inference$beta$p_values$enh_eigen_sign_flip)
+        }
+        }
+        if(inference.data.object@component[i]!=1){ # test on f was requested
+          p_value = statistics[length(statistics)]
+          
+          # add p-value in the right position
+          if(inference.data.object@type[i]==1){
+            inference$f$p_values$wald[[length(inference$f$p_values$wald)+1]] = p_value
+            inference$f$p_values$wald=as.list(inference$f$p_values$wald)
+          }
+          else if(inference.data.object@type[i]==3){
+            inference$f$p_values$eigen_sign_flip[[length(inference$f$p_values$eigen_sign_flip)+1]] = p_value
+            inference$f$p_values$eigen_sign_flip=as.list(inference$f$p_values$eigen_sign_flip)
+          }
+          else if(inference.data.object@type[i]==5){
+            inference$f$p_values$sign_flip[[length(inference$f$p_values$sign_flip)+1]] = p_value
+            inference$f$p_values$sign_flip=as.list(inference$f$p_values$sign_flip)
+          }
+        }
+        }
+      }
+    
+      if(inference.data.object@f_var==1){
+        f_variances = matrix(data = bigsol[[25]], nrow = length(observations), ncol = 1)
+        inference$f_var = f_variances
+      }  
+  
     reslist = list(fit.FEM = fit.FEM, PDEmisfit.FEM = PDEmisfit.FEM, solution = solution,
-                optimization  = optimization, time = time, bary.locations = bary.locations)
+                      optimization  = optimization, time = time, bary.locations = bary.locations, inference=inference)
+    }else{
+    reslist = list(fit.FEM = fit.FEM, PDEmisfit.FEM = PDEmisfit.FEM, solution = solution,
+                      optimization  = optimization, time = time, bary.locations = bary.locations)
+    }
     return(reslist)
   }
 }
